/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.commons.chain2.web.servlet;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.chain2.web.MapEntry;

/**
 * <p>Private implementation of <code>Map</code> for servlet parameter
 * name-values[].</p>
 *
 * @version $Id$
 */
final class ServletParamValuesMap implements Map<String, String[]>, Serializable {

    /** serialVersionUID */
    private static final long serialVersionUID = 5373913824627470557L;

    private transient HttpServletRequest request = null;

    ServletParamValuesMap(final HttpServletRequest req) {
        this.request = req;
    }

    @Override
    public void clear() {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean containsKey(final Object key) {
        return this.request.getParameter(toKey(key)) != null;
    }

    @Override
    public boolean containsValue(final Object value) {
        return values().contains(value);
    }

    @Override
    public Set<Entry<String, String[]>> entrySet() {
        Set<Entry<String, String[]>> set = new HashSet<>();
        for (final String key : Collections.list(this.request.getParameterNames())) {
            set.add(new MapEntry<>(key, this.request.getParameterValues(key), false));
        }
        return set;
    }

    @Override
    public boolean equals(final Object o) {
        if (o != this) {
            if (!ServletParamValuesMap.class.isInstance(o)) {
                return false;
            }
            ServletParamValuesMap map = (ServletParamValuesMap) o;
            return this.request.equals(map.request);
        }
        return true;
    }

    @Override
    public String[] get(final Object key) {
        return this.request.getParameterValues(toKey(key));
    }

    @Override
    public int hashCode() {
        return this.request.hashCode();
    }

    @Override
    public boolean isEmpty() {
        return size() < 1;
    }

    @Override
    public Set<String> keySet() {
        return new HashSet<>(Collections.list(this.request.getParameterNames()));
    }

    @Override
    public String[] put(final String key, final String[] value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void putAll(final Map<? extends String, ? extends String[]> map) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String[] remove(final Object key) {
        throw new UnsupportedOperationException();
    }

    @Override
    public int size() {
        return Collections.list(this.request.getParameterNames()).size();
    }

    @Override
    public Collection<String[]> values() {
        List<String[]> list = new ArrayList<>();
        for (final String key : Collections.list(this.request.getParameterNames())) {
            list.add(this.request.getParameterValues(key));
        }
        return list;
    }

    private String toKey(final Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        }
        return key.toString();
    }

}
